# This file is part of the digitalSTROM dsm-api library.
#
# Copyright (C) 2010 digitalSTROM.org, Schlieren, Switzerland
#
# digitalSTROM dsm-api library is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# digitalSTROM dsm-api library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with digitalSTROM dsm-api library. If not, see <http://www.gnu.org/licenses/>.

from PyGenerator import *
from string import Template

def loggerCallbackImplementation(self, type):
    s = Template("""    def ${callback_name}_callback(self, *arg):
        print datetime.now().strftime("[%Y-%m-%d %H:%M:%S.%f")[0:-3]+"]", "${callback_name}:", str(arg)
""")
    return s.substitute(callback_name = "%s_%s" % (self.name, type))
ApiCall.loggerCallbackImplementation = loggerCallbackImplementation

def loggerRequestResponseRegistration(self):
    s = Template("""        if not filter or filter.has_key($messageId):
            if not filter or not filter[$messageId] or $modifierId in filter[$messageId]:
                DsmApiRegisterPyCallback(self.handle, DS485_CONTAINER_REQUEST, $messageId, $modifierId, self.$request_func)
                DsmApiRegisterPyCallback(self.handle, DS485_CONTAINER_RESPONSE, $messageId, $modifierId, self.$response_func)
""")
    return s.substitute(messageId = self.message_id,
                        modifierId = self.modifier_id,
                        request_func = self.name + "_request_callback",
                        response_func = self.name + "_response_callback")
ApiCall.loggerRequestResponseRegistration = loggerRequestResponseRegistration

def loggerEventRegistration(self):
    s = Template("""        if not filter or filter.has_key($messageId):
            if not filter or not filter[$messageId] or $modifierId in filter[$messageId]:
                DsmApiRegisterPyCallback(self.handle, DS485_CONTAINER_EVENT, $messageId, $modifierId, self.$event_func)
""")
    return s.substitute(messageId = self.message_id,
                        modifierId = self.modifier_id,
                        event_func = self.name + "_event_callback")
ApiCall.loggerEventRegistration = loggerEventRegistration


def logger_start(self):
    s = ""
    for call in self.calls:
        s += call.loggerRequestResponseRegistration() + "\n"
    for event in self.events:
        s += event.loggerEventRegistration() + "\n"
    return s
CallGroup.logger_start = logger_start

def logger_callback_impl(self):
    s = ""
    for call in self.calls:
        s += call.loggerCallbackImplementation("request") + "\n"
        s += call.loggerCallbackImplementation("response") + "\n"
    for event in self.events:
        s += event.loggerCallbackImplementation("event") + "\n"
    return s
CallGroup.logger_callback_impl = logger_callback_impl

class ApiLoggerGenerator:
    def __init__(self, license, groups):
        self.groups = groups
        self.license = license
        
    def logger(self, logger_file):
        logger = open(logger_file, 'w')
        logger_str = "#!/usr/bin/python -u\n"
        logger_str += "\"\"\"" + self.license + "\n\"\"\"\n\n"
        logger_str += """from dsmapi2 import *
from datetime import datetime
import termios, fcntl, sys, os, time

# read a single keypress; from python FAQ
def getKey():
    fd = sys.stdin.fileno()
    oldterm = termios.tcgetattr(fd)
    newattr = termios.tcgetattr(fd)
    newattr[3] = newattr[3] & ~termios.ICANON & ~termios.ECHO
    termios.tcsetattr(fd, termios.TCSANOW, newattr)
    oldflags = fcntl.fcntl(fd, fcntl.F_GETFL)
    fcntl.fcntl(fd, fcntl.F_SETFL, oldflags | os.O_NONBLOCK)
    try:
        while 1:
            try:
                return sys.stdin.read(1)
            except IOError:
                time.sleep(0.2)
    finally:
        termios.tcsetattr(fd, termios.TCSAFLUSH, oldterm)
        fcntl.fcntl(fd, fcntl.F_SETFL, oldflags)

def waitForEscape():
    while 1:
        k = getKey()
        if k == '\x1b':
            return
        else:
            print k,

class DsmApiLogger:
    \"\"\"
    This Logger prints out the (filtered) dSM-API traffic present on the connected daemon.
    
    Arguments:
    handle -- daemon handle
    filter -- A dict() specifying which MessageID/ModifierID should be logged.
              None: Log all (default)
              {MessageID1: None, MessageID2: None}: Log all modifiers of MessageID1 and MessageID2
              {MessageID: (ModifierID1, ModifierID2)}: Log only ModifierID1 and ModifierID2 of MessageID
    \"\"\"
    def __init__(self, handle, filter=None):
        self.handle = handle
"""
        
        for group in self.groups:
            logger_str += group.logger_start()

        for group in self.groups:
            logger_str += group.logger_callback_impl()
            
        logger_str += """
if __name__ == "__main__":
    if len(sys.argv) > 2:
        print "I need the address of the daemon to work!"
        exit(0)

    if len(sys.argv) == 1:
        address = "tcp://localhost:8442"
    else:
        address = sys.argv[1]
    
    handle = DsmApiInitialize()
    if DsmApiOpen(handle, address, 1) != 0:
        print "Connecting to the daemon at", address, "failed."
        DsmApiCleanup(handle)
        exit(-1)
    
    logger = DsmApiLogger(handle)
    
    print 'Press Esc to quit.'
    waitForEscape()
    
    DsmApiClose(handle)
    DsmApiCleanup(handle)
"""
        logger.write(logger_str.encode('utf-8'))
        logger.close()
        